package org.jboss.forge.addon.git.ui;

import javax.inject.Inject;

import org.jboss.forge.addon.facets.constraints.FacetConstraint;
import org.jboss.forge.addon.git.facet.GitFacet;
import org.jboss.forge.addon.git.facet.GitIgnoreFacet;
import org.jboss.forge.addon.git.gitignore.GitIgnoreConfig;
import org.jboss.forge.addon.resource.FileResource;
import org.jboss.forge.addon.resource.ResourceFactory;
import org.jboss.forge.addon.ui.UICommand;
import org.jboss.forge.addon.ui.context.UIBuilder;
import org.jboss.forge.addon.ui.context.UIContext;
import org.jboss.forge.addon.ui.context.UIValidationContext;
import org.jboss.forge.addon.ui.input.UIInput;
import org.jboss.forge.addon.ui.metadata.WithAttributes;
import org.jboss.forge.addon.ui.result.Result;
import org.jboss.forge.addon.ui.result.Results;
import org.jboss.forge.addon.ui.util.Metadata;

@FacetConstraint({GitFacet.class, GitIgnoreFacet.class})
public class GitIgnoreSetupCommand extends AbstractGitCommand implements UICommand
{
   
   @Inject
   @WithAttributes(label = "Checkout directory", description = "Where should the gitignore" +
               " template repository be installed at?")
   private UIInput<FileResource<?>> checkoutDirInput;
   
   @Inject
   @WithAttributes(label = "Repository", description = "Do you want to provide a different repository" +
               " location for gitignore templates?")
   private UIInput<String> repositoryInput;
   
   @Inject
   private GitIgnoreConfig config;

   @Inject
   private ResourceFactory factory;

   @Override
   public Metadata getMetadata()
   {
      return super.getMetadata().name("GITIGNORE: Setup")
               .description("Create .gitignore files based on template files from https://github.com/github/gitignore.git.");
   }

   @Override
   public void initializeUI(UIBuilder builder) throws Exception
   {
      checkoutDirInput.setDefaultValue(getDefaultCheckoutDir());            
      repositoryInput.setDefaultValue(config.defaultRemoteRepository());
      
      builder.add(checkoutDirInput).add(repositoryInput);
   }

   private FileResource<?> getDefaultCheckoutDir()
   {
      return (FileResource<?>) factory.create(config.defaultLocalRepository());
   }
   
   @Override
   public void validate(UIValidationContext context)
   {
      FileResource<?> checkoutDir = checkoutDirInput.getValue();
      if (checkoutDir.exists()) 
      {
         if (!checkoutDir.isDirectory())
         {
            context.addValidationError(checkoutDirInput, 
                     "File " + checkoutDir + " is not a directory.");
         }
         if (!checkoutDir.listResources().isEmpty())
         {
            context.addValidationError(checkoutDirInput, 
                     "Directory " + checkoutDir + " is not empty");
         }
      }
   }

   @Override
   public Result execute(UIContext context) throws Exception
   {
      FileResource<?> checkoutDir = checkoutDirInput.getValue();
      if (!checkoutDir.exists()) 
      {
         checkoutDir.mkdirs();
      }
      
      config.setLocalRepository(checkoutDir.getFullyQualifiedName());
      config.setRemoteRepository(repositoryInput.getValue());

      return Results.success();
   }
 
   
}
